//
// array.cc
//
#include <iostream>
#include "common.h"

double* new_dvector( long nh ) 
{ 
   int na1;
   double* vec;    
   vec = new double[nh+1];
   if (!vec) errornew();
   for (na1 = 0; na1 <= nh; na1++){
       vec[na1] = 0.0;
   } 
   return vec;
}

long* new_lvector( long nh ) 
{ 
   int na1;
   long* vec;    
   vec = new long[nh+1];
   if (!vec) errornew();
   for (na1 = 0; na1 <= nh; na1++){
       vec[na1] = 0;
   } 
   return vec;
}
int* new_ivector( long nh ) 
{ 
   int na1;
   int* vec;    
   vec = new int[nh+1];
   if (!vec) errornew();
   for (na1 = 0; na1 <= nh; na1++){
       vec[na1] = 0;
   } 
   return vec;
}

double** new_dmatrix(long nr, long nc)
{
   int na1,na2;
   double** mat;
   
   mat = new double* [nr+1];
   if (!mat) errornew();
   
   for(na1 = 0; na1 <= nr; na1++){
      mat[na1] = new double [nc+1];
      if(!mat[na1]) errornew();
   }
   for (na1 = 0; na1 <= nr; na1++)
       for (na2 = 0; na2 <= nc; na2++)
          mat[na1][na2] = 0.0;  
   return mat; 	   
}

int** new_imatrix(long nr, long nc)
{
   int na1,na2;
   int** mat;
   
   mat = new int* [nr+1];
   if (!mat) errornew();
   
   for(na1 = 0; na1 <= nr; na1++){
      mat[na1] = new int [nc+1];
      if(!mat[na1]) errornew();
   }
   for (na1 = 0; na1 <= nr; na1++)
       for (na2 = 0; na2 <= nc; na2++)
          mat[na1][na2] = 0;
   return mat;	     
}

double*** new_dtensor3(long nr, long nc, long nh)
{
   int na1,na2,na3;
   double*** mat;
   
   mat = new double** [nr+1];
   if (!mat) errornew();
   
   for(na1 = 0; na1 <= nr; na1++){
      mat[na1] = new double* [nc+1];
      if(!mat[na1]) errornew();
      for(na2 = 0; na2 <= nc; na2++){
         mat[na1][na2] = new double [nh+1];
	 if(!mat[na1][na2]) errornew();
      }   
   }
   for (na1 = 0; na1 <= nr; na1++)
       for (na2 = 0; na2 <= nc; na2++)
           for (na3 = 0; na3 <= nh; na3++)     
               mat[na1][na2][na3] = 0.0;
   return mat; 	          
}

int*** new_itensor3(long nr, long nc, long nh)
{
   int na1,na2,na3;
   int*** mat;
   
   mat = new int** [nr+1];
   if (!mat) errornew();
   
   for(na1 = 0; na1 <= nr; na1++){
      mat[na1] = new int* [nc+1];
      if(!mat[na1]) errornew();
      for(na2 = 0; na2 <= nc; na2++){
         mat[na1][na2] = new int [nh+1];
	 if(!mat[na1][na2]) errornew();
      }   
   }
   for (na1 = 0; na1 <= nr; na1++)
       for (na2 = 0; na2 <= nc; na2++)
           for (na3 = 0; na3 <= nh; na3++)     
               mat[na1][na2][na3] = 0;
   return mat; 	          
}

void free_vector( double* v ) { delete[] v; }
void free_vector( long*   v ) { delete[] v; }
void free_vector( int*    v ) { delete[] v; }

void free_matrix( double **m, long nr ) {
    int na1;
    for (na1 = 0; na1 <= nr; na1++)
        delete [] m[na1];
    delete[] m;	    
}

void free_matrix( int **m, long nr ) {
    int na1;
    for (na1 = 0; na1 <= nr; na1++)
        delete [] m[na1];
    delete[] m;
}

void free_tensor(double*** m, long nr, long nc) {
    int na1,na2;
    for( na1 = 0; na1 <= nr; na1++){
       for(na2 = 0; na2 <= nc; na2++){
          delete [] m[na1][na2];
       }
       delete [] m[na1];
    }   	  
    delete[] m;
}

void free_tensor(int*** m, long nr, long nc) {
    int na1,na2;
    for( na1 = 0; na1 <= nr; na1++){
       for(na2 = 0; na2 <= nc; na2++){
          delete [] m[na1][na2];
       }
       delete [] m[na1];
    }   	  
    delete[] m;
}    

void errornew() {
    cout << "Not enough memory to allocate the array!\n" << endl;
    exit(-1);
    return;
}    
// end array.cc
