#include "common.h"
#include "base.h"
#include "sla.h"
#include "funk.h"
#include "edmonds.h"
#include "mesh.h"
#include "eigenval.h"
#include "prep.h"

#define EPS 3.0e-11 /* EPS is the relative precision for gauss integration. */
#define MAXIT 60

//const double NA = 6.02214179;  //e23mole-1
//const double Boltzman = 8.617343e-11;  //MeV*K-1
//const double emolar = 5.485799e-4; //g*mol-1

double fermi_dirac_e(double, double);

double fermi_dirac_e(double Eel,double lambda_e)
{
  double S_e;

  S_e = 1.0/(exp((Eel - lambda_e)/(Boltzman*temp*1e9))+1);

  return S_e;

}


double elambdafuncsum(double temp, double lambda_e)
{
 double func1, func2, func3, func,lambda1,lambda2,lambda3;
 double elambdafunc(double,double,int,double,double);

// lambda1=lambda_e-1;
// lambda2=lambda_e+1;
// lambda3=lambda_e+15;
 lambda1=0;
 lambda2=lambda_e + 15*temp*Boltzman*1e9;
 lambda3=lambda_e + 15*temp*Boltzman*1e9;
 if (lambda1<0) 
 {  
  lambda1 =0;
 }
 func1 = elambdafunc(0,lambda1,0,temp,lambda_e);
 func2 = elambdafunc(lambda1,lambda2,80,temp,lambda_e);
 func3 = elambdafunc(lambda2,lambda3,0,temp,lambda_e);
 func = func1 + func2 + func3;
 return func;
}

double elambdafunc(double Eelmin,double Eelmax,int n_gauss,double temp, double lambda_e)
{
 double factor;
 double energyc, EkT_e,EkT_p, fermi_emp, p_electron,sum;
// double Eelmax = 60;
 
 int k_gauss;
// int n_gauss= 50;
// double energy[n_gauss], w_energy[n_gauss];
 double *energy = new double[n_gauss];
 double *w_energy = new double[n_gauss];
 void gauleg(double, double, double *, double *, int);

 sum = 0;

 factor = pow((1/hqc),3.0)/pow(pi,2)/NA *1.0e16; //e39 from hqc - e23 from NA
    
 gauleg(-1,1,energy,w_energy,n_gauss-1);
 
 for ( k_gauss=1; k_gauss <= n_gauss-1; k_gauss ++)
{
 energyc = (Eelmax-Eelmin)/2*energy[k_gauss] + (Eelmax+Eelmin)/2 + emass; 
 
 EkT_e = (energyc -lambda_e)/(Boltzman*temp*1e9);
 EkT_p = (energyc +lambda_e)/(Boltzman*temp*1e9);

 fermi_emp = 1.0/(exp(EkT_e)+1) - 1.0/(exp(EkT_p)+1);

 p_electron = sqrt(energyc*energyc - emass*emass);
  
 sum += fermi_emp*energyc*p_electron*w_energy[k_gauss];

// cout <<energyc<<" " <<(fermi_emp*energyc*p_electron) << endl;
// cout << "sum= " << sum << endl;
}

 sum = (Eelmax-Eelmin)/2*sum*factor;
// cout << sum << endl;
 delete[] energy;
 delete[] w_energy;
 return sum;
}


double rtsec(double x1, double x2, double xacc,double rhoYe,double temp)
{
// void nrerror(char error_test[]);
  double elambdafuncsum(double,double);
 int j;
 double fl,f,dx,swap,xl,rts;
 
// rhoYe = rhoYe/emolar;

 fl= (elambdafuncsum(temp,x1)-rhoYe);
 f = (elambdafuncsum(temp,x2)-rhoYe);
 if (fabs(fl) < fabs(f))
 {
  rts = x1;
  xl=x2;
  swap=fl;
  fl=f;
  f=swap;
 }
 else
 {
  xl=x1;
  rts=x2;
 }
 for(j=1;j<=MAXIT;j++)
 {
  dx=(xl-rts)*f/(f-fl);
  xl=rts;
  fl=f;
  rts += dx;
  f=(elambdafuncsum(temp,rts)-rhoYe);
  if (fabs(dx)< xacc || f==0.0) return rts;
 }
  cout<<"Maximum number of interations exceeded in rtsec:" <<MAXIT<< endl;
// nrerror("Maximum number of interations exceeded in rtsec");
 exit(0);
 return 0.0;
}

