
#ifndef _common_h
#define _common_h

using namespace std;

#include <iostream>
#include <iomanip>
//#include <stdiostream.h>
#include <fstream>
#include <math.h>
#include <stdlib.h>
#include <stdarg.h> 
#include <string.h>

// Definition for constants 
 
const double pi = 3.141592653589793;
const double zero = 0.0;
const double one = 1.0;
const double two = 2.0;
const double half = 0.5;
const double third = 1.0/3.0;
const double eps = 0.001;
const double hqc = 197.328284;
const double emass = 0.511003;
const double alpha = 7.2972e-3;
const double MN = 939.0;

const double NA = 6.02214179;  //e23mole-1
const double Boltzman = 8.617343e-11;  //MeV*K-1
const double emolar = 5.485799e-4; //g*mol-1

// Definition for Routine spline

#define YP 1.e31

extern double* new_dvector(long nl);
extern void    free_vector(double* v);

extern long*   new_lvector(long nl);
extern void    free_vector(long* v);

extern int*    new_ivector(long nl);
extern void    free_ivector(int* v);

extern double** new_dmatrix(long nr, long nc);
extern void     free_matrix(double** m, long nr);

extern int**    new_imatrix(long nr, long nc);
extern void     free_matrix(int** m, long nr);

extern double*** new_dtensor3(long nr, long nc, long nh);
extern void     free_tensor(double*** m, long nr, long nc);

extern int***   new_itensor3(long nr, long nc, long nh);
extern void     free_tensor(int*** m, long nr, long nc);

extern void errornew();

#define double1(var) double *var
#define double2(var) double **var
#define double3(var) double ***var
#define int1(var) int *var
#define int2(var) int **var
#define int3(var) int ***var

#define double1init(var1,var2) { var1=new_dvector(var2); }
#define double2init(var1,var2,var3) { var1=new_dmatrix(var2,var3); }
#define double3init(var1,var2,var3,var4) { var1=new_dtensor3(var2,var3,var4); }

#define double1del(var1) { free_vector(var1); }  
#define double2del(var1,var2) { free_matrix(var1,var2); } 
#define double3del(var1,var2,var3) { free_tensor(var1,var2,var3); }  

#define int1init(var1,var2) { var1=new_ivector(var2); }
#define int2init(var1,var2,var3) { var1=new_imatrix(var2,var3); }
#define int3init(var1,var2,var3,var4) { var1=new_itensor3(var2,var3,var4); }

#define int1del(var1) { free_vector(var1); } 
#define int2del(var1,var2) { free_matrix(var1,var2); }
#define int3del(var1,var2,var3) { free_tensor(var1,var2,var3); } 

#endif
