#include "common.h"
#include "sla.h"
#include "funk.h"
#include "edmonds.h"

//
// berechnet 3j-Symbole fuer ganz- und halbzahlige Argumente            
//


double wigner(double j1, double j2, double j3, double m1,
	      double m2, double m3)

{

    int i0, i1, i2, i3, i4, i5, i6, i7, n1, n2;
    double wigret = zero;

    

    i3 = nint0(j2+m2);
    i4 = nint0(j3-j2+m1);
    i7 = nint0(j3-m3);
    
    if (i7 != (i3+i4))
      return wigret;
    
    i0 = nint0(j1+j2+j3+1);
    i1 = nint0(j1+j2-j3);
    i2 = nint0(j1-m1);
    i5 = nint0(j3-j1-m2);
    i6 = nint0(j3+m3);
    
    n1 = max0(3, 0, -i4, -i5);
    n2 = min0(3, i1, i2, i3); 
    

    if (n1 > n2)
      return wigret;
    

    for (int n = n1; n <= n2; n++)
    {
	wigret = wigret+iv[n]*
	         fi[n]*fi[i1-n]*fi[i2-n]*fi[i3-n]*fi[i4+n]*fi[i5+n];
    }
    
    

    wigret *= iv[i2+i4+i6]*
	     wfi[i0]*wf[i1]*wf[i2+i4]*wf[i3+i5]*
	     wf[i1+i4]*wf[i2]*wf[i3]*wf[i1+i5]*wf[i6]*wf[i7];
	
    
    
    return wigret;

}


//
// berechnet 3j-Symbole fuer ganzzahlige Argumente            
//


double wigner(int j1, int j2, int j3, int m1,
	      int m2, int m3)

{

    int i0, i1, i2, i3, i4, i5, i6, i7, n1, n2;
    double wigret = zero;


    if ((m1+m2+m3) != 0)
      return wigret;
    

    i3 = j2+m2;
    i4 = j3-j2+m1;
    
    
    i0 = j1+j2+j3+1;
    i1 = j1+j2-j3;
    i2 = j1-m1;
    i5 = j3-j1-m2;
    
    n1 = max0(3, 0, -i4, -i5);
    n2 = min0(3, i1, i2, i3); 
    

    if (n1 > n2)
      return wigret;
    

    for (int n = n1; n <= n2; n++)
    {
	wigret += iv[n]*
	         fi[n]*fi[i1-n]*fi[i2-n]*fi[i3-n]*fi[i4+n]*fi[i5+n];
    }
    
    

    wigret *= iv[i2+i3]*
	     wfi[i0]*wf[i1]*wf[i2+i4]*wf[i3+i5]*
	     wf[i2]*wf[j1+m1]*wf[j2-m2]*wf[i3]*wf[j3-m3]*wf[j3+m3];
	
    
    
    return wigret;

}

double dsq0(double a, double b, double c)

{

    int i1, i2, i3, i4;
    double dsqret = zero;


    i1 = nint0(a+b-c);
    i2 = nint0(a-b+c);
    i3 = nint0(-a+b+c);
    i4 = nint0(a+b+c)+1;

    if ((i1 < 0) || (i2 < 0) || (i3 < 0))
      return dsqret;

    dsqret = wf[i1]*wf[i2]*wf[i3]*wfi[i4];

    return dsqret;
}


double dsq0(int a, int b, int c)

{

    int i1, i2, i3, i4;
    double dsqret = zero;


    i1 = a+b-c;
    i2 = a-b+c;
    i3 = -a+b+c;
    i4 = a+b+c+1;

    if ((i1 < 0) || (i2 < 0) || (i3 < 0))
      return dsqret;
    

    dsqret = wf[i1]*wf[i2]*wf[i3]*wfi[i4];

    return dsqret;
}




//
// berechnet 6j-Symbole fuer ganz- und halbzahlige Argumente      
// (Formel (6.3.7) aus Edmonds p.99)
//


double racah(double j1, double j2, double j3,
	     double l1, double l2, double l3)

{
    int i1, i2, i3, i4, i5, i6, i7, n1, n2;
    double racahret = zero, trs;
    


    trs = dsq0(j1, j2, j3)*dsq0(j1, l2, l3)*dsq0(l1, j2, l3)*
          dsq0(l1, l2, j3);
 
    if (trs == 0)
      return racahret;


    i1 = nint0(j1+j2+j3);
    i2 = nint0(j1+l2+l3);
    i3 = nint0(l1+j2+l3);
    i4 = nint0(l1+l2+j3);
    i5 = nint0(j1+j2+l1+l2);
    i6 = nint0(j2+j3+l2+l3);
    i7 = nint0(j3+j1+l3+l1);


    n1 = max0(4, i1, i2, i3, i4);
    n2 = min0(3, i5, i6, i7);

    if (n1 > n2)
      return racahret;



    for (int n = n1; n <= n2; n++) {

	racahret = racahret + iv[n]*fak[n+1]*
	           fi[n-i1]*fi[n-i2]*fi[n-i3]*fi[n-i4]*
		   fi[i5-n]*fi[i6-n]*fi[i7-n];
	
    }

    racahret = racahret * trs;

    return racahret;
}




//
// berechnet 6j-Symbole nur fuer ganzzahlige Argumente      
// (Formel (6.3.7) aus Edmonds p.99)
//


double racah(int j1, int j2, int j3,
	     int l1, int l2, int l3)

{
    int i1, i2, i3, i4, i5, i6, i7, n1, n2;
    double racahret = zero, trs;
    

    trs = dsq0(j1, j2, j3)*dsq0(j1, l2, l3)*dsq0(l1, j2, l3)*
          dsq0(l1, l2, j3);
 
    if (trs == 0)
      return racahret;

    
    i1 = j1+j2+j3;
    i2 = j1+l2+l3;
    i3 = l1+j2+l3;
    i4 = l1+l2+j3;
    i5 = j1+j2+l1+l2;
    i6 = j2+j3+l2+l3;
    i7 = j3+j1+l3+l1;

    n1 = max0(4, i1, i2, i3, i4);
    n2 = min0(3, i5, i6, i7);

    if (n1 > n2)
      return racahret;

        
    for (int n = n1; n <= n2; n++) {

	racahret = racahret + iv[n]*fak[n+1]*
	           fi[n-i1]*fi[n-i2]*fi[n-i3]*fi[n-i4]*
		   fi[i5-n]*fi[i6-n]*fi[i7-n];
	
    }

    racahret = racahret * trs;

    return racahret;
}

// enthaelt spezielle Racahfunktionen

extern double racah1(int, int, int, int, int, int);

extern double racah2(int, int, int, int, int, int);

extern double sdsq10(int, int, int);

extern double sdsq0(int, int, int);

extern double sdsh0(int, int, int);



double sdsq10(int a, int b, int c) {

// Definition nur fuer racah1() gueltig !!
// von a und b muss jeweils 1/2 abgezogen werden

    int i1, i2, i3, i4;
    double sdsq1ret = zero;


    i1 = a+b-c-1;
    i2 = a-b+c;
    i3 = -a+b+c;
    i4 = a+b+c;


    if ((i1 < 0) || (i2 < 0) || (i3 < 0))
      return sdsq1ret;



    sdsq1ret = wf[i1]*wf[i2]*wf[i3]*wfi[i4];

    return sdsq1ret;
}
    

double sdsq0(int a, int b, int c) {

// Definition wie in racah.cc!!


    int i1, i2, i3, i4;
    double sdsqret = zero;

    
    i1 = a+b-c;
    i2 = a-b+c;
    i3 = -a+b+c;
    i4 = a+b+c+1;

    if ((i1 < 0) || (i2 < 0) || (i3 < 0))
      return sdsqret;

  

    sdsqret = wf[i1]*wf[i2]*wf[i3]*wfi[i4];


    return sdsqret;
}


double sdsh0(int a, int b, int c) {

// Definition nur fuer racah2() gueltig!
// von b und c muessen jeweils 1/2 abgezogen werden


    int i1, i2, i3, i4;
    double sdshret = zero;



    i1 = a+b-c;
    i2 = a-b+c;
    i3 = -a+b+c-1;
    i4 = a+b+c;

    if ((i1 < 0) || (i2 < 0) || (i3 < 0))
      return sdshret;




    sdshret = wf[i1]*wf[i2]*wf[i3]*wfi[i4];


    return sdshret;
}





double racah1(int j1, int j2, int j, int l1, int l2, int l) {

// Achtung: j1,j2,l1 und l2 wurden als j1+1/2, j2+1/2 etc. uebergeben!


    int i1, i2, i3, i4, i5, i6, i7, n1, n2;
    double racah1ret = zero, trs;
    
   
    trs = sdsq10(j1,j2,j)*sdsq10(j1,l2,l)*sdsq10(l1,j2,l)*
          sdsq10(l1,l2,j);
 
    if (trs == 0)
      return racah1ret;



    i1 = j1+j2+j-1;
    i2 = j1+l2+l-1;
    i3 = l1+j2+l-1;
    i4 = l1+l2+j-1;
    i5 = j1+j2+l1+l2-2;
    i6 = j2+j+l2+l-1;
    i7 = j+j1+l+l1-1;

    n1 = max0(4,i1,i2,i3,i4);
    n2 = min0(3,i5,i6,i7);

    if (n1 > n2)
      return racah1ret;

    for (int n = n1; n <= n2; n++) {
	
	racah1ret = racah1ret+iv[n]*fak[n+1]*
	            fi[n-i1]*fi[n-i2]*fi[n-i3]*fi[n-i4]*
		    fi[i5-n]*fi[i6-n]*fi[i7-n];
    }


    return (trs*racah1ret);

}



double racah2(int j1, int j2, int j3, int l1, int l2, int l3) {

// Achtung: l1,l2 und l3 wurden als l1+1/2, l2+1/2 etc. uebergeben!



    double trs, racah2ret = zero;
    int i1, i2, i3, i4, i5, i6, i7, n1, n2;



    trs = sdsq0(j1,j2,j3)*sdsh0(j1,l2,l3)*sdsh0(j2,l1,l3)*
          sdsh0(j3,l1,l2);


 
    if (trs == 0)
      return racah2ret;


    i1 = j1+j2+j3;
    i2 = j1+l2+l3-1;
    i3 = l1+j2+l3-1;
    i4 = l1+l2+j3-1;
    i5 = j1+j2+l1+l2-1;
    i6 = j2+j3+l2+l3-1;
    i7 = j3+j1+l3+l1-1;

    n1 = max0(4,i1,i2,i3,i4);
    n2 = min0(3,i5,i6,i7);


    for (int n = n1; n <= n2; n++) {

	racah2ret = racah2ret + iv[n]*fak[n+1]*
	            fi[n-i1]*fi[n-i2]*fi[n-i3]*fi[n-i4]*
		    fi[i5-n]*fi[i6-n]*fi[i7-n];
    }


    return (trs*racah2ret);

}

double s9jslj(int, int, int, int, int, int);



//
// berechnet das 9j-Symbol    1/2   1/2  S
//                             l1    l2  L      
//                             j1    j2  J
//  
//

double s9jslj(int s, int l1, int l2, int l,
	      int j1, int j2, int j)

// Achtung: j1 und j2 wurden als j1=j1+1/2 und j2=j2+1/2 uebergeben!
// j bleibt gleich!

{

    int la;
    double s9ret = zero, s0, s1, s2;


    // Test der Parameter
    // alle Drehimpulse positiv?

    if ((l1 < 0) || (l2 < 0) || (l < 0) || (j1 < 0) 
	|| (j2 < 0) || (j < 0)) {
	
	cout << "Fehler bei lj-Argumenten von s9jslj()!\n";
	exit(1); 
    }

    // Dreiecksungleichungen erfuellt?


    if ( (l < abs0(l1-l2)) || (l > (l1+l2)) 
	 || (j < abs0(j1-j2)) || (j > (j1+j2-1))
	 || (l1 < abs0(j1-1)) || (l1 > j1)
	 || (l2 < abs0(j2-1)) || (l2 > j2)
         || (j < abs0(l-s)) || (j > (l+s)))
      return s9ret;






    
    switch (s) {
      case 0:
// l=j !!

	s9ret = iv[j1+l2+j]*sqi[4*j+2]*
	        racah2(j,l2,l1,1,j1,j2);
	
	break;

      case 1:


	if (j == l)
	{
	    la = l+1;
	    s0 = -sq[2*l+1]*sq[l+1]*sqi[l]*shi[1];
	    s2 = iv[j1+l2+la]*racah2(l1,l2,l,j2,j1,1)/(4*l+2);

	}
    

	else
	{
	    la = max0(2,l,j);
	    s0 = sq[la]*shi[1];
	    s2 = zero;
	}
	
	s1 = -racah2(l1,l2,l,1,la,j2)*racah1(j1,j2,j,la,1,l1);
	s9ret = (s1-s2)*s0;

       	break;


      default:

	cout << "Fehler bei s-Argument von s9jslj()!\n" ;
	exit(1);
    }

    return s9ret;

}

    
