#include "common.h"
#include "sla.h"
#include "funk.h"
#include "base.h"
#include "edmonds.h"

// needed for integrals from 0 to "r"

double simps2(double *f, int n, double h) {


    double simps2ret = zero;
    int nst;


    if (n == 0)
      return simps2ret;
    
    

    if ((n%2) == 0) 
    {	
    
	for (nst = 0; nst <= (n/2-1); nst++)
	  simps2ret += f[2*nst]+4*f[2*nst+1]+f[2*nst+2];
    
	simps2ret *= h*third;
	return simps2ret;
    }
    
    else if ((n%3) == 0)
    {
	for (nst = 0; nst <= (n/3-1); nst++)
	 simps2ret += f[3*nst]+3*f[3*nst+1]+3*f[3*nst+2]+f[3*nst+3];

	simps2ret *= 3./8.*h;
	return simps2ret;
    }
    
 
    else 
    {
	switch(n) 
	{
	    	
	  case 1:
	   
	    simps2ret = h/2.*(f[0]+f[1]);
	    return simps2ret;
	
	  default:
	    

	    for (nst = 0; nst <= ((n-3)/2-1); nst++)
	      simps2ret += f[2*nst]+4*f[2*nst+1]+f[2*nst+2];
    
	    simps2ret *= h*third;

	    simps2ret += 3*h/8.*(f[n-3]+3*f[n-2]+3*f[n-1]+f[n]);    	
	    return simps2ret;
	}
	

    }
    
}

 

// needed for integrals from r to "infinity"


double simps3(double *f, int nstart, int ndiff, double h) {


    double simps3ret = zero;
    int nst;
        



    if (ndiff == 0)
      return simps3ret;


    if ((ndiff%2) == 0) 
    {	
	
	
	for (nst = 0; nst <= (ndiff/2-1); nst++)
	{
	    	    
	    simps3ret += f[2*nst+nstart]+4*f[2*nst+1+nstart]+f[2*nst+2+nstart];
	     
	}
	

	simps3ret *= h*third;
	return simps3ret;
    }
    
    else if ((ndiff%3) == 0)
    {
	for (nst = 0; nst <= (ndiff/3-1); nst++)
	 simps3ret += f[3*nst+nstart]+3*f[3*nst+1+nstart]
	   +3*f[3*nst+2+nstart]+f[3*nst+3+nstart];

	simps3ret *= 3./8.*h;
	return simps3ret;
    }
    
 
    else 
    {
	switch(ndiff) 
	{
	    	
	  case 1:
	   
	    simps3ret = h/2.*(f[0+nstart]+f[1+nstart]);
	    return simps3ret;
	
	  default:
	    

	    for (nst = 0; nst <= ((ndiff-3)/2-1); nst++)
	      simps3ret += f[2*nst+nstart]+4*f[2*nst+1+nstart]
		+f[2*nst+2+nstart];
    
	    simps3ret *= h*third;

	    simps3ret += 3*h/8.*(f[ndiff-3+nstart]+3*f[ndiff-2+nstart]
			      +3*f[ndiff-1+nstart]+f[ndiff+nstart]); 	    
	    return simps3ret;
	}	
    }    
}

double bode(double *f, int n, double h) 
{
    double boderet = zero;
    int nst,i;
    
    if((n-1)%4 == 0)
    {
       for (nst = 1; nst <= (n-1)/4; nst++)
       {
           i = (nst-1)*4;
           boderet += 32.0*f[i+4]+12.0*f[i+3]+32.0*f[i+2]+14.0*f[i+1];
       }
       boderet -= 7.0*f[1];
       boderet -= 7.0*f[n];
       boderet = boderet*h/22.5;
    }
    else
    {
       cout << "qmesh-1 not multiple of 4!\n";
       exit(-1);
    }
    return boderet;
}       

//     reduced matrix element of Y_J in jj-coupling                 
//              < l1,j1 || Y_J || l2,j2 >
double yjj(int j1, int l1, int j2, int l2, int jot)
{
    double yjj_ret = zero;
    if (iv[abs0(l1+l2+jot)] != 1)
    {
        return yjj_ret;
    }
    else
    {
        yjj_ret = -  iv[j2+j1+jot] * // YF.Niu 2010.3.5. phaseI
                sq[j1]*sq[j2]*iv[j1]*sqd[jot]*
	        wigner(j1-0.5,jot,j2-0.5,-0.5,0.0,0.5)/sqrt(pi);
        return yjj_ret;
    }
}

//     reduced matrix element of [sigma * Y_L]_J in jj-coupling                 
//     < l1,j1 || [sigma Y_L]_J || l2,j2 >
double syjj(int j1, int l1, int j2, int l2, int l, int jot)
{
    double syjj_ret = zero;
    
    syjj_ret = iv[l1+l2+j1+j2-1+l+jot]* // YF.Niu 2010.3.5. phaseI
               sq[6]*sq[j1]*sq[j2]*iv[l1]*sqd[jot]*sqd[l]*sqd[l1]*sqd[l2]*
               wigner(l1,l,l2,0,0,0)*s9jslj(1,l1,l2,l,j1,j2,jot)/sqrt(pi); 
    return syjj_ret;
} 



