#include "common.h"
#include "base.h"
#include "sla.h"
#include "edmonds.h"
#include "mesh.h"
#include "prep.h"
#include "nr.h"
#include "nrutil.h"
#include "funk.h"

//-----------------------------------------------------------------------
double mered_MJL_sigma(int st1, int l1, int j1,
                 int st2, int l2, int j2, int L, int J,
                 double Kappa, int isign)
{
 double mel = zero;
 double fact = zero;

 if(L >= 0)
 {
  fact = iv[l1]
         //iv[l2+L+j1+j2-1+J] 
         * sqrt(6.0*(2*j2)*(2*j1)*(2*l2+1)*(2*l1+1)
                       * (2*L+1)*(2*J+1)/(4*pi))
                       * wigner(l1,L,l2,0,0,0)
                       * s9jslj(1,l1,l2,L,j1,j2,J);  
  }

  if(fact != 0.0)
  mel = fact * sinradjl(st1,l1,st2,l2,L,Kappa,0,1,0,isign);
//  cout << "fact = " << fact << endl;
//  cout <<" radial = " << sinradjl(st1,l1,st2,l2,L,Kappa,0,1,0,isign) << endl;

 return mel;
}

//-----------------------------------------------------------------------
double mered_MJL_nabla(int st1, int l1, int j1,
                 int st2, int l2, int j2, int L, int J,
                 double Kappa,int isign)
{

 double mel = zero;
 double fact0 = zero;
 double fact1 = zero;
 double fact2 = zero;
 double me_rad1 = zero;
 double me_rad2 = zero;

 // check the phase in fact0

 fact0 = iv[j1+j2+1+J]
         *iv[l1+j2+1] * sqrt((double)((2*j1)*(2*j2))) 
                     * racah((double)l1,j1-half,half,j2-half,(double)l2,(double)J)
                     * sqrt((2*l2+1)*(2*l1+1)*(2*L+1)*(2*J+1)/(4*pi));

 fact1 = racah(L,1,J,l2,l1,l2+1) 
         * (wigner(l1,L,l2+1,0,0,0)/wigner(l2+1,1,l2,0,0,0))
         * (l2+1.0)/(2*l2+1.0);
 

 if(l2 != 0)
 fact2 = racah(L,1,J,l2,l1,l2-1)
         * (wigner(l1,L,l2-1,0,0,0)/wigner(l2-1,1,l2,0,0,0))
         * l2/(2*l2+1.0);

 if(fact0 != 0.0)
 {
  if(fact1 != 0.0)
    me_rad1 = sinradjl(st1,l1,st2,l2,L,Kappa,-l2,0,1,isign);

  if(fact2 != 0.0)
    me_rad2 = sinradjl(st1,l1,st2,l2,L,Kappa,l2+1,0,1,isign);
 }

 mel = fact0 * (fact1 * me_rad1 + fact2 * me_rad2);

 return mel;
}

//-----------------------------------------------------------------------
double mered_MJ_nabla_sigma(int st1, int l1, int j1,
                    int st2, int l2, int j2, int J,
                    double Kappa,int isign)
{
 double mel = zero;
 double fact0 = zero;
 double fact1 = zero;
 double fact2 = zero;
 double me_rad1 = zero;
 double me_rad2 = zero;

// fact0 = iv[1-J] * sqrt(6*(2*j2)*(2*j1)*(2*l2+1)*(2*l1+1)*(2*J+1)/(4*pi));

 fact0 = iv[l1+l2+j1+j2] *
         iv[1+J] * sqrt(6*(2*j2)*(2*j1)*(2*l2+1)*(2*l1+1)*(2*J+1)/(4*pi)); //YF.Niu 2010.3.5. iv[n] n >=0

 fact1 = racah(half,half,1.0,(double)l2,(double)l2+1,j2-half)
         * racah(j1-half,j2-half,(double)J,(double)l2+1,(double)l1,half)
         * (wigner(l1,J,l2+1,0,0,0)/wigner(l2+1,1,l2,0,0,0))
         * (l2+1.0)/(2.0*l2+1.0);
 
 if(l2 != 0)
 fact2 = racah(half,half,1.0,(double)l2,(double)l2-1,j2-half)
         *racah(j1-half,j2-half,(double)J,(double)l2-1,(double)l1,half)
         *(wigner(l1,J,l2-1,0,0,0)/wigner(l2-1,1.0,l2,0,0,0))
         *(l2/(2.0*l2+1.0));

 if(fact0 != 0.0)
 {
  if(fact1 != 0.0)
    me_rad1 = sinradjl(st1,l1,st2,l2,J,Kappa,-l2,0,1,isign);

  if(fact2 != 0.0)
    me_rad2 = sinradjl(st1,l1,st2,l2,J,Kappa,l2+1,0,1,isign);
 }

 mel = fact0 * (fact1 * me_rad1 + fact2 * me_rad2);

 return mel;

}

//-----------------------------------------------------------------------
double mered_MJ(int st1, int l1, int j1,
                int st2, int l2, int j2, int J, 
                double Kappa,int isign)
{
 double mel = zero;
 double mefact = zero;

 mefact = iv[j1+j2+J+1]*
          iv[j2+J] * sqrt((2*j2)*(2*j1)*(2*l2+1)*(2*l1+1)*(2*J+1)/(4*pi))
          * wigner(l1,J,l2,0,0,0) * racah((double)l1, j1-half, half, j2-half, (double)l2, (double)J); 

 if(mefact != 0.0) 
   mel = mefact * sinradjl(st1,l1,st2,l2,J,Kappa,0,1,0,isign);
 
 return mel;
}

//-----------------------------------------------------------------------
double mered_OMEGAJ(int st1, int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa, int isign)
{
 double mel = zero;
 double mefact = zero;
 double mefact0 = zero;
 double mefact1 = zero;
 double mefact2 = zero;
 double me_rad1 = zero;
 double me_rad2 = zero;

 mefact0 = iv[l1+l2+j1+j2] * 
           iv[l1]*sqrt((2*j1)*(2*j2)*(2*l1+1)*(2*J+1)/(4*pi));

 mefact1 = -1.0 * deltafunk(j2,l2+1) * sqrt((double)(2*l2+3)) 
         * racah((double)J,(double)l1,(double)l2+1,half,j2-half,j1-half) * wigner(l1,J,l2+1,0,0,0);   

 if (2*l2-1 > 0)
 mefact2 = deltafunk(j2,l2) * sqrt((double)(2*l2 - 1))
         * racah((double)J,(double)l1,(double)l2-1,half,j2-half,j1-half) * wigner(l1,J,l2-1,0,0,0);  

 if(mefact0 != 0.0)
 {
  if(mefact1 != 0.0)
    me_rad1 = sinradjl(st1,l1,st2,l2,J,Kappa,-l2,0,1,isign);

  if(mefact2 != 0.0)
    me_rad2 = sinradjl(st1,l1,st2,l2,J,Kappa,l2+1,0,1,isign);

 }

 mel = (hqc/Kappa) * mefact0 * (mefact1 * me_rad1 + mefact2 * me_rad2);

 return mel;

}

double mered_DELTAJ(int st1,int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa, int isign)
{
 double meldelta=zero;

 meldelta = (hqc/Kappa)*mered_MJL_nabla(st1,l1,j1,
                 st2,l2,j2,J,J,
                 Kappa, isign);

 return meldelta;
} 

double mered_DELTAJ_prime(int st1,int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa,int isign)
{
 double meldeltap=zero;

 meldeltap = (hqc/Kappa)
            *(-sqrt((double)J/(double)(2.0*J+1.0))
               *mered_MJL_nabla(st1,l1,j1,st2,l2,j2,J+1,J,Kappa,isign)
              +sqrt((double)(J+1.0)/(double)(2.0*J+1.0))
               *mered_MJL_nabla(st1,l1,j1,st2,l2,j2,J-1,J,Kappa,isign)); 
             
 return meldeltap; 
}

double mered_SIGMAJ(int st1,int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa,int isign)
{
 double melsigma;

 melsigma = mered_MJL_sigma(st1,l1,j1,st2,l2,j2,J,J,Kappa,isign);

 return melsigma;

}

double mered_SIGMAJ_prime(int st1,int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa, int isign)
{
 double melsigmap;

 melsigmap = -1.0*sqrt((double)J/(double)(2.0*J+1.0))
                 *mered_MJL_sigma(st1,l1,j1,st2,l2,j2,J+1,J,Kappa,isign)
             + sqrt((double)(J+1.0)/(double)(2.0*J+1.0))
                 *mered_MJL_sigma(st1,l1,j1,st2,l2,j2,J-1,J,Kappa,isign);

 return melsigmap;

}

double mered_SIGMAJ_twoprimes(int st1,int l1, int j1,
                   int st2, int l2, int j2, int J,
                   double Kappa,int isign)
{
 double melsigmapp;

 melsigmapp = sqrt((double)(J+1.0)/(double)(2.0*J+1.0))
                *mered_MJL_sigma(st1,l1,j1,st2,l2,j2,J+1,J,Kappa, isign)
            + sqrt((double)J/(double)(2.0*J+1.0))
                *mered_MJL_sigma(st1,l1,j1,st2,l2,j2,J-1,J,Kappa, isign); 

 return melsigmapp;

}

