/*----------------------------------------------------------
 
FINITE TEMPERATURE RELATIVISTIC RANDOM PHASE APPROXIMATION

------------------------------------------------------------*/
#include "common.h"
#include "base.h"
#include "sla.h"
#include "funk.h"
#include "edmonds.h"
#include "mesh.h"
#include "eigenval.h"
#include "prep.h"

int matnum, matnumsav;
int nmass;
double1(rmass);
int lmax;
int nonlin;
double gcoul;
int natural_parity;
double gpi,ampi;


void ftchrpa(int jtot, int iparity, int xyorder)
    {
    int pa1, pa2;
    long count;
    unsigned long int nmatsav = 0;
    int str1, str3, tr1, tr3;
    double vh, vp;
    int n, m, pottype;
    double darg;
    int i, l;
    int iii;
    long mat_tot;
    fstream xrpafile;
    fstream yrpafile;
    fstream erpafile;
    fstream c_erpafile;
/*
    char nxrpafile[15];
    char nyrpafile[15];
    char nerpafile[15];
    char nc_erpafile[15];
*/

    char *nxrpafile=new char[15];
    char *nyrpafile=new char[15];
    char *nerpafile=new char[15];
    char *nc_erpafile=new char[15];

    int ab_read = 0;
    double v0,vs, vpp;
    int st1,st2,t1,t2;
    double a_mat_ph, a_mat_pp, b_mat_ph, b_mat_pp;

    time_t tm1, tm2;

//    int jtot, iparity;
    
    Eigenvalue *ev;
    
    // Coulomb-Coupling (e*e)/(4*pi) = 1/alpha
    gcoul = (1./137.03602);
    gpi = 1.43371;
    ampi = 138.0;
   
    // Reading input parameters from start.dat

    sprintf(nxrpafile,"xrpa%d_%d.bin",jtot,iparity);
    sprintf(nyrpafile,"yrpa%d_%d.bin",jtot,iparity);
    sprintf(nerpafile,"erpa%d_%d.bin",jtot,iparity);
    sprintf(nc_erpafile,"c_erpa%d_%d.bin",jtot,iparity);

    paramet("start.dat");

    if(tempMeV >= 0.1 && tempMeV <=0.6) qptresh = 0.2;
    if(tempMeV > 0.6 && tempMeV <=1.2) qptresh = 0.02; 
    if(tempMeV > 1.2)  qptresh = 0.05;
//    if(tempMeV >= 2.4) qptresh = 0.1;
//    if(rhoYe >= 1.0e13) 
//    {
//     qptresh = 0.2;
//    }

    if (lorchange == 1) 
    {
	cout << "### only calculating new lorentz-curves ###" << endl;
	cout << "### other parameters except of excitation properties are " 
	  << "ignored ###" << endl; 
	goto lorenzneu;
    }

/*    
    if (xyread == 1) 
    {
	cout << "*** first reading rpa-results for further calculation ***" 
	  << endl;
    }
*/
    // check of parameters

    if ( ((xyread == 1) && (calc == 1))
	|| ((xyread == 1) && (xyprint == 1)))
    {
	cout << "wrong combination of parameters, please correct" << endl;
	cout << "xyread: " << transdens << endl;
	cout << "xyprint: " << xyprint << endl;
	cout << "calc: " << calc << endl;
	cout << "program is terminating" << endl;
	exit(1);
    }
    
    if (xyread != 1) 
    {
	if ((calc > 1)) 
	{
	    cout << "parameters not valid, please correct" << endl;
	    cout << "calc: " << calc << endl;
	    cout << "program is terminating" << endl;
	    exit(1);	
	}
    }
    
    // three mesons 
    nmass = 3; 

    // allocation

    double1init(rmass,nmass);

     j = jtot;
     parity = iparity; //add for EC
     cout << "j = " << j << " parity = " << parity << endl;

     if(xyorder == 0) {  
                       xyread = 1;
                       calc = 0;
                       xyprint = 0;
                       matprint = 0;
                      }
     else if (xyorder == 1) {
           xyread = 0;
           calc = 1;
           xyprint = 1;
           matprint = 1;
          } 

    // maximal l-Value
    
    lmax = j + 1;
    
    
    // reading the wave functions and occupation probabilites from rpa.wel

    if(jtot == 0 && iparity == 1 && xyorder == 1)
    {
     // allocation

     double1init(rmass,nmass);

     incanon();

     prep(); 

     deriv();           //add for EC

     wfinter(); 
      
//    cout << "... preparing the Gaussian-integration on double-mesh" << endl;

     testint();

//    cout << "... testing mesh" << endl;

      gfv();

//  Values of rmass

    rmass[1] = amsig/hqc;
    rmass[2] = amome/hqc;
    rmass[3] = amrho/hqc;

//    cout << "... creating quasi-particle levels" << endl;

    qplevel();

    }
 
 
    // natural parity?

    if (((j+parity+1)%2) == 0)
      natural_parity = 1;
    else 
      natural_parity = 0;
    
//    cout << "... creating quasi-particle pairs" << endl;
   
    cout << "qptresh = " << qptresh << endl;

    phpair(j, parity);

//    cout << "... writing out quasiparticle pairs" << endl;

//    qppairout(j, parity, "pair.out");    
    	
//    cout << npair << " pairs found!" << endl;
//    cout << npair_ph << "/" << npair_ah << 
//	  " ph-/ah-pairs" << endl;
		  
//    cout << "... calculating bessel function for pairing" << endl;
//    qpbess(); 
       
//    cout << "... starting rpa calculation" << endl;
    rpa(npair);
   
//    cout << " proba " << endl;

    int jtest;

    jtest = 1;
 
//    cout << " jtest-half = " << jtest-half << endl;
 
//    cout << "nmesh = " << nmesh << endl;
//    cout << "nonlin = " << nonlin << endl;


    // calculation starts
    if (calc == 1) 
    {
//	cout << "full calculation" << endl;

	// calculation of modified spherical besselfunctions
	// for nonlinear terms of sigmamesons

        propa();
	jlyl();		

	count = 0;

	// total number of matrixelements to be calculated
	
	mat_tot = npair*(npair+1);

//	cout << "... calculating RPA matrices" << endl;
     	for (pa1 = 1; pa1 <= npair; pa1++)
	{
	    for (pa2 = 1; pa2 <= pa1; pa2++)
	    {	
	       // ph interaction
               v0 = matcon0(pa1,pa2);
	       vs = matcons(pa1,pa2);
/*	       
               // pp interaction
	       if ((respair == 1) && (iph[pa1][4] == 1) && (iph[pa2][4] == 1))
	       {  vpp = qpmatpair(pa1,pa2);
		 // cout << pa1 << " " << pa2 << " " << vpp << endl;
	       }  
               else
	          vpp = 0.0;
*/		  
               // quasiparticle A matrix	           
	       a_mat_ph = qpmataph(pa1,pa2,v0,vs);
	       
//	       a_mat_pp = qpmatapp(pa1,pa2,vpp);
	       arpa[pa1][pa2] = a_mat_ph; // + a_mat_pp;
	//  cout << pa1 << " " << pa2 << "\t" <<  arpa[pa1][pa2] << endl;
	       
	       // quasiparticle B matrix
	       b_mat_ph = qpmatbph(pa1,pa2,v0,vs);
//	       b_mat_pp = qpmatbpp(pa1,pa2,vpp);
//               b_mat_ph = 0;
	       brpa[pa1][pa2] = b_mat_ph; // + b_mat_pp;	       
	       	   
	       count += 2;
/*		
	       if (count%1000 == 0) 
	       {
		//    cout << "pa1 = " << pa1 << " pa2 = " << pa2 << 
		      " " << ((double)count/(double)mat_tot)*100. << endl;
	       }		
*/
	    }
	}	 

//        cout << "... RPA matrices generated" << endl;
	      	
	msym(); 	

/*
        if (matprint == 1)
        {
            mout(arpa,npair,6,"rel. RPA-Matrix A","arpa.con");
            mout(brpa,npair,6,"rel. RPA-Matrix B","brpa.con");

        }

	
	mbout(arpa,npair,"arpa.bin");
	mbout(brpa,npair,"brpa.bin");
*/
    }
/*    else 
    {
	mbin(arpa,npair,"arpa.bin");
	mbin(brpa,npair,"brpa.bin");
	ab_read = 1;	
    }
*/

    if (xyread == 1)
      goto xy_read;
        	
    double1init(erpas,2*npair-1);
    double1init(c_erpas,2*npair-1);
    double2init(rpa_full,2*npair-1,2*npair-1);
    double2init(xyrpa,2*npair-1,2*npair-1);


    for (i = 0; i < npair; i++) 
    {
	for (l = 0; l < npair; l++) 
	{
	    rpa_full[i][l] = arpa[i+1][l+1];
	    rpa_full[i+npair][l] = -brpa[i+1][l+1];
	    rpa_full[i][l+npair] = brpa[i+1][l+1];
	    rpa_full[i+npair][l+npair] = -arpa[i+1][l+1];
	}
    }

//    cout << "...calculating eigenvalues ; diagonalizing " << 2*npair
//	  << " x " << 2*npair << " RQRPA matrix" << endl;	    
    ev = new Eigenvalue(rpa_full,2*npair);
	
    erpas = ev->getRealEigenvalues();
    c_erpas = ev->getImagEigenvalues();
    xyrpa = ev->getV();

    delete ev;

    double1init(erpa,2*npair);
    double1init(c_erpa,2*npair);
    double2init(xrpa,npair,2*npair);
    double2init(yrpa,npair,2*npair);

    for (i = 0; i < 2*npair; i++)
    {
	erpa[i+1] = erpas[i];
	c_erpa[i+1] = c_erpas[i];
	    
	for (l = 0; l < npair; l++)
	{		
	    xrpa[l+1][i+1] = xyrpa[l][i];
	    yrpa[l+1][i+1] = xyrpa[l+npair][i];
	}
    }

    double2del(xyrpa,2*npair-1);
    double1del(erpas);
    double1del(c_erpas);
    double2del(rpa_full,2*npair-1);

//    cout << "sorting rpa-solution" << endl;
    rpasort(npair,xrpa,yrpa,erpa,c_erpa);

    int energyc, pairc;
    double1(charge);
    double1init(charge, 2*npair); 
    
 /*     
    for(energyc = 86; energyc <= 86; energyc++)
    {
      
      for(pairc =1; pairc <= npair; pairc++)
      {
     cout << "xrpa[" << pairc <<"] = " << xrpa[pairc][energyc] << " yrpa[" << pairc <<"] = " << yrpa[pairc][energyc] << endl;
 
      charge[energyc] += xrpa[pairc][energyc]*xrpa[pairc][energyc] - yrpa[pairc][energyc]*yrpa[pairc][energyc];
      }
      cout << "charge =" << charge[energyc] << endl;
      cout << "erpa[" <<energyc <<"]" <<"  = " << erpa[energyc] << endl;
    }



    
    for (i = 1; i < 2*npair+1; i++)
    {
     cout << "E = " << erpa[i] << endl;
    }
*/

/* 
    // prints out x- and y- and energy-matrices
    if ( xyprint == 1) 
    {
	cout << "saving x- and y-matrices and energies on disk" << endl;
		  
	mbout(xrpa,npair,2*npair,"xrpa.bin");
	mbout(yrpa,npair,2*npair,"yrpa.bin");
	vbout(erpa,2*npair,"erpa.bin");
	vbout(c_erpa,2*npair,"c_erpa.bin");	
    }
*/

//    cout << "testing rpa-solution" << endl;    
    rpaprobe(npair,arpa,brpa,xrpa,yrpa,erpa);

      // prints out x- and y- and energy-matrices
    if ( xyprint == 1)
    {
        cout << "saving x- and y-matrices and energies on disk" << endl;

        mbout(xrpa,npair,2*npair,nxrpafile);
        mbout(yrpa,npair,2*npair,nyrpafile);
        vbout(erpa,2*npair,nerpafile);
        vbout(c_erpa,2*npair,nc_erpafile);
    }

/*
    for(energyc = 86; energyc <= 86; energyc++)
    {
      for(pairc =1; pairc <= npair; pairc++)
      {
     cout << "xrpa[" << pairc <<"] = " << xrpa[pairc][energyc] << " yrpa[" << pairc <<"] = " << yrpa[pairc][energyc] << endl;

      charge[energyc] += xrpa[pairc][energyc]*xrpa[pairc][energyc] - yrpa[pairc][energyc]*yrpa[pairc][energyc];
      }
      cout << "charge =" << charge[energyc] << endl;
      cout << "erpa[" <<energyc <<"]" <<"  = " << erpa[energyc] << endl;
    }
*/

//    cout << "calculating excitation-strength" << endl;	
/*    
    excstr(erpa,xrpa,yrpa,npair,j,"excm.out","excp.out",
	   "curve.out","neudens.out","prodens.out",
	   "totdens.out", "lorm.out", 
	   "lorp.out","pskal.out", "pvec.out", 
	   "strength.out", 
	   lorswidth, lorvwidth, 0);	      
*/ 
    // excitations without interaction
    if (hartree == 1)
    {	
        double1init(erpas,2*npair-1);
        double1init(c_erpas,2*npair-1);
        double2init(rpa_full,2*npair-1,2*npair-1);
        double2init(xyrpa,2*npair-1,2*npair-1);

	for (i = 0; i <= npair; i++)  
	{
	    for (l = 0; l <= npair; l++)
	    {
		arpa[i][l] = zero;
		brpa[i][l] = zero;
	    }
	}	

	// just h11 matrix
	for (pa1 = 1; pa1 <= npair; pa1++)
	{
	   for (pa2 = 1; pa2 <= npair; pa2++)
	   {
               arpa[pa1][pa2] = qpmataph(pa1,pa2,0.0,0.0);
	       brpa[pa1][pa2] = 0.0;
	   }   
	}      	      

	for (i = 0; i < npair; i++) 
	{
	    for (l = 0; l < npair; l++) 
	    {
		rpa_full[i][l] = arpa[i+1][l+1];
		rpa_full[i+npair][l] = -brpa[i+1][l+1];
		rpa_full[i][l+npair] = brpa[i+1][l+1];
		rpa_full[i+npair][l+npair] = -arpa[i+1][l+1];
	    }
	}

	cout << "calculating unperturbed eigenvalues" << endl;	    
	ev = new Eigenvalue(rpa_full,2*npair);
	
	erpas = ev->getRealEigenvalues();
	c_erpas = ev->getImagEigenvalues();
	xyrpa = ev->getV();

        delete ev;
    
	for (i = 0; i < 2*npair; i++)
	{
	    erpa[i+1] = erpas[i];
	    c_erpa[i+1] = c_erpas[i];
	    
	    for (l = 0; l < npair; l++)
	    {		
		xrpa[l+1][i+1] = xyrpa[l][i];
		yrpa[l+1][i+1] = xyrpa[l+npair][i];
	    }
	}

        double2del(xyrpa,2*npair-1);
        double1del(erpas);
        double1del(c_erpas);
        double2del(rpa_full,2*npair-1);

	cout << "sorting hartree-solution" << endl;
	rpasort(npair,xrpa,yrpa,erpa,c_erpa);
		
       	
	cout << "testing hartree-solution" << endl;    
	//rpaprobe(npair,arpa,brpa,xrpa,yrpa,erpa);

	cout << "calculating hartree excitation-strength" << endl;	
/*	excstr(erpa,xrpa,yrpa,npair,j,"harexcm.out","harexcp.out", 
	       "harcurve.out","harneudens.out","harprodens.out",
	       "hartotdens.out", "harlorm.out", 
	       "harlorp.out", "harpskal.out", "harpvec.out", 
	       "harstrength.out", hlorswidth, hlorvwidth, 1);
 */ 
    }
    
    goto ende;
    
    

    // calculating new lorenz-curves
  lorenzneu:
    if (lorchange == 1) 
    {	
	lorneu("pskal.out", "pvec.out", "nlorskal.out",
	       "nlorvec.out");
	
	if (hartree == 1)
	  lorneu("harpskal.out", "harpvec.out",
		 "nharlorskal.out", "nharlorvec.out");

	goto ende;	
    }
    

	
    // this part reads x and y and energies from disk for further  
    // calculations
  xy_read:
  
    // test if x and y and energies are on disk-file

    xrpafile.open(nxrpafile, ios::in);
    yrpafile.open(nyrpafile, ios::in);
    erpafile.open(nerpafile, ios::in);
    c_erpafile.open(nc_erpafile, ios::in);
    
    if ((!xrpafile) || (!yrpafile) || (!erpafile) ||
	(!c_erpafile)) {
	cout << "x- and y-matrices and energies are not all on disk" << endl;
	cout << "please make a new run and save these matrices" << endl;
	cout << "program is terminating" << endl;
	exit(1);
    }
//    else 
//      cout << "reading x- and y-matrices and energies" << endl;
   
    ndim_o_c = 2*npair;
 
    double2init(xrpa,npair,2*npair);
    double2init(yrpa,npair,2*npair);
    double1init(erpa,2*npair);
    double1init(c_erpa,2*npair);

    mbin(xrpa,npair,2*npair,nxrpafile);
    mbin(yrpa,npair,2*npair,nyrpafile);
    vbin(erpa,2*npair,nerpafile);
    vbin(c_erpa,2*npair,nc_erpafile);	

/*    
    if (ab_read == 1) 
    {	
	if (xyprobe == 1) 
	{	    
	    cout << "testing rpa-solution after reading matrices" << endl;    
	    rpaprobe(npair,arpa,brpa,xrpa,yrpa,erpa);
	}
	else 
	  ndim_o_c = 2*npair;	
	
    }
    else 
    {
	cout << "a and b-matrices not read in" << endl;
	cout << "program is terminating" << endl;
	exit(1);
    }
*/
/*
    if (exccalc == 1)
    {
	cout << "### calculating excitation-strength ###" << endl;	

	excstr(erpa,xrpa,yrpa,npair,j,"excskal.out","excvec.out",
	       "curve.out","neudens.out","prodens.out",
	       "totdens.out", "lorskal.out", 
	       "lorvec.out","pskal.out", "pvec.out", 
	       "strength.out", 
	       lorswidth, lorvwidth, 0);
  
    }
*/

  ende:

  delete[] nxrpafile;
  delete[] nyrpafile;
  delete[] nerpafile;
  delete[] nc_erpafile;

  double2del(arpa,npair);
  double2del(brpa,npair);
  

 if(calc == 1)
 {
  double3del(yjl,maxq,npair);
  double2del(yjl_pi,maxq);
  double1del(ddrho);
  double1del(ddpi);
 }

 //   cout << "program terminated without errors" << endl;
  cout << " ";
    
 
}










